<?php

declare(strict_types = 1);

namespace Drupal\migrate_dc\Plugin\migrate\process;

use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\ProcessPluginBase;
use Drupal\migrate\Row as MigrateRow;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @\Drupal\migrate\Annotation\MigrateProcessPlugin(
 *   id = "migrate_dc_ensure_uuid"
 * )
 */
class MigrateDcEnsureUuid extends ProcessPluginBase implements ContainerFactoryPluginInterface {

  protected UuidInterface $uuidGenerator;

  protected string $uuidPattern = '/^[a-z0-9]{8}-[a-z0-9]{4}-[a-z0-9]{4}-[a-z0-9]{4}-[a-z0-9]{12}$/i';

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('uuid'),
    );
  }

  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    UuidInterface $uuidGenerator,
  ) {
    $this->uuidGenerator = $uuidGenerator;

    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public function transform($value, MigrateExecutableInterface $migrate_executable, MigrateRow $row, $destination_property) {
    return $this->isUuid($value) ?
      $value
      : $this->uuidGenerator->generate();
  }

  protected function isUuid($value): bool {
    return preg_match($this->uuidPattern, (string) $value) === 1;
  }

}
