<?php

declare(strict_types = 1);

namespace Drupal\Tests\migrate_dc\Kernel\Plugin\migrate\process;

use Drupal;
use Drupal\migrate\MigrateExecutable;
use Drupal\migrate\MigrateMessageInterface;
use Drupal\migrate\Row as MigrateRow;
use Drupal\migrate_dc\Plugin\migrate\process\MigrateDcUriTransformator;
use Drupal\node\Entity\Node;
use Drupal\node\Entity\NodeType;
use Drupal\KernelTests\KernelTestBase;

/**
 * @group migrate_dc
 *
 * @covers \Drupal\migrate_dc\Plugin\migrate\process\MigrateDcUriTransformator
 */
class MigrateDcUriTransformatorTest extends KernelTestBase implements MigrateMessageInterface {

  protected static $modules = [
    'system',
    'field',
    'filter',
    'text',
    'user',
    'node',
    'migrate',
    'migrate_dc',
  ];

  protected string $bundle = 'page';

  protected MigrateRow $row;

  protected \Drupal\migrate\MigrateExecutable $migrateExecutable;

  /**
   * @var null
   */
  protected $destinationProperty;

  protected \Drupal\migrate\Plugin\MigrationPluginManager $migrationPluginManager;

  protected array $configuration = [];

  protected string $pluginId = '';

  protected array $pluginDefinition = [];

  protected \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager;

  protected \Drupal\node\NodeTypeInterface $nodeType;

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function setUp(): void {
    parent::setUp();

    $values = [
      'type' => $this->bundle,
      'name' => 'Page',
    ];
    $this->nodeType = NodeType::create($values);
    $this->nodeType->save();

    $this->installEntitySchema('node');
    $this->installEntitySchema('user');
    $this->installConfig(static::$modules);
    $this->row = new MigrateRow();
    $this->destinationProperty = NULL;

    $container = Drupal::getContainer();
    $this->entityTypeManager = $container->get('entity_type.manager');

    $this->migrationPluginManager = Drupal::service('plugin.manager.migration');
    /** @var \Drupal\migrate\Plugin\Migration $migration */
    $migration = $this->migrationPluginManager->createStubMigration([]);
    $this->migrateExecutable = new MigrateExecutable(
      $migration,
      // @todo Message instance.
      $this,
    );
  }

  /**
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testTransform(): void {
    /** @var \Drupal\node\Entity\Node $node */
    $node = Node::create([
      'type' => $this->nodeType->bundle(),
      'title' => 'Test node',
      'uid' => 1,
    ]);
    $node->save();

    $expected = 'entity:node/' . $node->id();
    $value = 'entity.uuid:node/' . $node->uuid();

    $processor = new MigrateDcUriTransformator(
      $this->configuration,
      $this->pluginId,
      $this->pluginDefinition,
      $this->entityTypeManager
    );

    $actual = $processor->transform(
      $value,
      $this->migrateExecutable,
      $this->row,
      $this->destinationProperty
    );

    static::assertSame($expected, $actual);
  }

  /**
   * {@inheritdoc}
   */
  public function display($message, $type = 'status') {
    static::assertTrue($type === 'status', $message);
  }

}
